using System;
using System.Windows.Forms;
using System.Collections.Generic;
using System.Text;

//using System.Diagnostics;
using System.Runtime.InteropServices;

using AKSYSTech.Device.Adapters.Interface;

using ManyLeaves.Infrastructure.Interface;

namespace AKSYSTech.Device.Adapters
{
	[Flags]
	public enum InvokeDirection
	{
		None = 0,
		In = 1, // methods and properties
		Out = 2 // events
	}

	public class Invoke : IDevice
	{
		private Form _form = null;
		private IDevice _invokee = null;
		private InvokeDirection _direction = InvokeDirection.None;

		#region Events

		public event EventHandler<EventArgs<Position[]>> PositionChanged;
		public event EventHandler<EventArgs<DeviceState>> DeviceStateChanged;
		public event EventHandler<EventArgs<AutoState>> AutoStateChanged;
		public event EventHandler<EventArgs<GCodeState>> GCodeStateChanged;
		public event EventHandler<EventArgs<String>> StatusMessageChanged;
		public event EventHandler<EventArgs<Rate[]>> RateChanged;
        public event EventHandler<EventArgs<Switch[]>> SwitchChanged;


		#endregion

		#region Constructor/Dispose

		public Invoke(IDevice invokee, InvokeDirection direction, Form form)
		{
			_form = form;
			_invokee = invokee;
			_direction = direction;
		}

		public void Load()
		{
			_invokee.PositionChanged += new EventHandler<EventArgs<Position[]>>(_invokee_PositionChanged);
			_invokee.DeviceStateChanged += new EventHandler<EventArgs<DeviceState>>(_invokee_DeviceStateChanged);
			_invokee.AutoStateChanged += new EventHandler<EventArgs<AutoState>>(_invokee_AutoStateChanged);
			_invokee.GCodeStateChanged += new EventHandler<EventArgs<GCodeState>>(_invokee_GCodeStateChanged);
			_invokee.StatusMessageChanged += new EventHandler<EventArgs<String>>(_invokee_StatusMessageChanged);
			_invokee.RateChanged += new EventHandler<EventArgs<Rate[]>>(_invokee_RateChanged);
            _invokee.SwitchChanged += new EventHandler<EventArgs<Switch[]>>(_invokee_SwitchChanged);


			_invokee.Load();
		}

		public void Close()
		{
			_invokee.Close();
		}

		public void Dispose()
		{
			_invokee.Dispose();
			_invokee = null;
		}

		#endregion

		#region Events

		private Boolean InvokeEvents { get { return (_direction & InvokeDirection.Out) == InvokeDirection.Out; } }
		private Boolean InvokeMethods { get { return (_direction & InvokeDirection.In) == InvokeDirection.In; } }

		private void _invokee_PositionChanged(object sender, EventArgs<Position[]> e)
		{
			if (_form.InvokeRequired && this.InvokeEvents)
			{
				_form.Invoke(new EventHandler<EventArgs<Position[]>>(_invokee_PositionChanged), new Object[] { sender, e });
			}
			else
			{
				this.PositionChanged(sender, e);
			}
		}

		private void _invokee_DeviceStateChanged(object sender, EventArgs<DeviceState> e)
		{
			if (_form.InvokeRequired && this.InvokeEvents)
			{
				_form.Invoke(new EventHandler<EventArgs<DeviceState>>(_invokee_DeviceStateChanged), new Object[] { sender, e });
			}
			else
			{
				this.DeviceStateChanged(sender, e);
			}
		}

		private void _invokee_AutoStateChanged(object sender, EventArgs<AutoState> e)
		{
			if (_form.InvokeRequired && this.InvokeEvents)
			{
				_form.Invoke(new EventHandler<EventArgs<AutoState>>(_invokee_AutoStateChanged), new Object[] { sender, e });
			}
			else
			{
				this.AutoStateChanged(sender, e);
			}
		}

		private void _invokee_GCodeStateChanged(object sender, EventArgs<GCodeState> e)
		{
			if (_form.InvokeRequired && this.InvokeEvents)
			{
				_form.Invoke(new EventHandler<EventArgs<GCodeState>>(_invokee_GCodeStateChanged), new Object[] { sender, e });
			}
			else
			{
				this.GCodeStateChanged(sender, e);
			}
		}

		private void _invokee_StatusMessageChanged(object sender, EventArgs<String> e)
		{
			if (_form.InvokeRequired && this.InvokeEvents)
			{
				_form.Invoke(new EventHandler<EventArgs<String>>(_invokee_StatusMessageChanged), new Object[] { sender, e });
			}
			else
			{
				this.StatusMessageChanged(sender, e);
			}
		}

		private void _invokee_RateChanged(object sender, EventArgs<Rate[]> e)
		{
			if (_form.InvokeRequired && this.InvokeEvents)
			{
				_form.Invoke(new EventHandler<EventArgs<Rate[]>>(_invokee_RateChanged), new Object[] { sender, e });
			}
			else
			{
				this.RateChanged(sender, e);
			}
		}

        private void _invokee_SwitchChanged(object sender, EventArgs<Switch[]> e)
        {
            if (_form.InvokeRequired && this.InvokeEvents)
            {
                _form.Invoke(new EventHandler<EventArgs<Switch[]>>(_invokee_SwitchChanged), new Object[] { sender, e });
            }
            else
            {
                this.SwitchChanged(sender, e);
            }
        }

		#endregion

		#region Properties

		private delegate void GCodeDelegate(String gcode);
		private void GCodeMethod(String gcode) { this.GCode = gcode; }

		public String GCode
		{
			set
			{
				if (_form.InvokeRequired && this.InvokeMethods)
				{
					_form.Invoke(new GCodeDelegate(GCodeMethod), new Object[] { value });
				}
				else
				{
					_invokee.GCode = value;
				}
			}
		}

		private delegate void AutoStateDelegate(AutoState status);
		private void AutoStateMethod(AutoState status) { this.AutoState = status; }

		public AutoState AutoState
		{
			set
			{
				if (_form.InvokeRequired && this.InvokeMethods)
				{
					_form.Invoke(new AutoStateDelegate(AutoStateMethod), new Object[] { value });
				}
				else
				{
					_invokee.AutoState = value;
				}
			}
		}

		#endregion

		#region Methods

        private delegate void RateDoubleDelegate(RateName name, Double value);

        public void Rate(RateName name, Double value)
		{
			if (_form.InvokeRequired && this.InvokeMethods)
			{
                _form.Invoke(new RateDoubleDelegate(Rate), new Object[] { name, value });
			}
			else
			{
                _invokee.Rate(name, value);
			}
		}

        private delegate void RateDirectionDelegate(RateName name, JogDirection direction);

        public void Rate(RateName name, JogDirection direction)
		{
			if (_form.InvokeRequired && this.InvokeMethods)
			{
                _form.Invoke(new RateDirectionDelegate(Rate), new Object[] { name, direction });
			}
			else
			{
                _invokee.Rate(name, direction);
			}
		}

        private delegate Double RateDelegate(RateName name);

        public Double Min(RateName name)
        {
            if (_form.InvokeRequired && this.InvokeMethods)
            {
                return (Double)_form.Invoke(new RateDelegate(Min), new Object[] { name });
            }
            else
            {
                return _invokee.Min(name);
            }
        }

        public Double Max(RateName name)
        {
            if (_form.InvokeRequired && this.InvokeMethods)
            {
                return (Double)_form.Invoke(new RateDelegate(Max), new Object[] { name });
            }
            else
            {
                return _invokee.Max(name);
            }
        }


        private delegate void SwitchPositionDelegate(SwitchName name, SwitchPosition position);

        public void Switch(SwitchName name, SwitchPosition position)
        {
            if (_form.InvokeRequired && this.InvokeMethods)
            {
                _form.Invoke(new SwitchPositionDelegate(Switch), new Object[] { name, position });
            }
            else
            {
                _invokee.Switch(name, position);
            }
        }

        

		private delegate void JogDelegate(Axis axis, JogDirection direction);

		public void Jog(Axis axis, JogDirection direction)
		{
			if (_form.InvokeRequired && this.InvokeMethods)
			{
				_form.Invoke(new JogDelegate(Jog), new Object[] { axis, direction });
			}
			else
			{
				_invokee.Jog(axis, direction);
			}
		}

		#endregion
	}
}
